/***************************************************************************
                          config.cpp  -  description
                             -------------------
    begin                : Sat Feb 9 2002
    copyright            : (C) 2002 by funthing (Matthias Zisler)
    email                : matthias@zislers.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "config.h"
#include "global_types.h"
#include "resources.h"
#include <fstream>
#include <sys/stat.h>

#ifdef WIN32
#else
#include <dirent.h>
#endif

int Config::music=true;
int Config::music_vol=2;
int Config::maxl=100;
int Config::intro=true;
int Config::skill=2;
int Config::full_screen=true;
int Config::speed=3;
int Config::fps=false;
int Config::last_level=1;
unsigned int Config::ms=0;
int Config::keys[18]={1,4,23,19,17,5,26,24,3,85,87,89,86,88,90,82,83,84};

#ifdef WIN32
static CL_String custom_path="";
#else
static CL_String custom_path=(CL_String)(getenv("HOME"))+"/.auriferous/";
#endif

CL_String Config::last_levelset="default";
CL_String Config::filename=custom_path+"auriferous.cfg";
vector <CL_String> Config::map_samples;

bool Config::get_music()
{
	return music;
}
void Config::set_music(bool m)
{
	music=m;
}
bool Config::get_intro()
{
	return intro;
}
void Config::set_intro(bool i)
{
	intro=i;
}
bool Config::get_fps()
{
	return fps;
}
void Config::set_fps(bool f)
{
	fps=f;
}
bool Config::get_full_screen()
{
	return full_screen;
}
void Config::set_full_screen(bool f)
{
	full_screen=f;
}
int Config::get_music_vol()
{
	return music_vol;
}
void Config::set_music_vol(int v)
{
	music_vol=v;
}
int Config::get_speed()
{
	return speed;
}
void Config::set_speed(int s)
{
	speed=s;
}
int Config::get_skill()
{
	return skill;
}
void Config::set_skill(int s)
{
	skill=s;
}
int Config::get_last_level()
{
	return last_level;
}
void Config::set_last_level(int l)
{
	last_level=l;
}
CL_String Config::get_last_levelset()
{
	return last_levelset;
}
void Config::set_last_levelset(CL_String l)
{
	last_levelset=l;
}


void Config::init()
{
#ifdef WIN32
;//nothing to do :)
#else
	CL_String path=(CL_String)(getenv("HOME"))+"/.auriferous";
	if(!opendir(path))
	mkdir(path, 0755);

	path=(CL_String)(getenv("HOME"))+"/.auriferous/levels";
	if(!opendir(path))
	mkdir(path, 0755);

	path=(CL_String)(getenv("HOME"))+"/.auriferous/levels/custom";
	if(!opendir(path))
	mkdir(path, 0755);
#endif


	FILE* configfile=fopen(filename, "r");
	if(!configfile)
	{
    deinit(); //writes a default config
  }
	else
	{
		char ch[2]={0,0};
		last_levelset="";
		
		while ((ch[0]=fgetc(configfile)) !=10)
		{
			last_levelset+=ch;
		}

		fscanf(configfile, "%d\n", &last_level);
		fscanf(configfile, "%d\n", &music);
		fscanf(configfile, "%d\n", &music_vol);
		fscanf(configfile, "%d\n", &intro);
		fscanf(configfile, "%d\n", &skill);
		fscanf(configfile, "%d\n", &full_screen);
		fscanf(configfile, "%d\n", &speed);
		fscanf(configfile, "%d\n", &fps);

		int val;
		for(int i=0; i<18; i++)
		{
			fscanf(configfile, "%d\n", &val);
			keys[i]=val;
		}
	}

	map_samples.push_back(CL_String("default"));
	try
	{
		CL_DirectoryScanner scan;

		if(scan.scan((custom_path+"levels/"), "*"))
		{
			scan.next();
			scan.next();

			while(scan.next())
			{
				if(scan.is_directory())
				map_samples.push_back(scan.get_name().c_str());
			}
		}
	}
	catch(CL_Error err)
	{
		cout<<"no extra maps"<<endl;
	}

  for(unsigned int i=0; i<map_samples.size(); i++)
  {
    if(map_samples[i]==last_levelset)
    ms=i;
  }

}
void Config::next_map()
{
	if(ms>0)
	last_levelset=map_samples[--ms];
}
void Config::prev_map()
{
	if(ms+1<map_samples.size())
	last_levelset=map_samples[++ms];
}
void Config::deinit()
{
	ofstream out(filename.get_string());

	out<<last_levelset<<endl;
	out<<last_level<<endl;
	out<<music<<endl;
	out<<music_vol<<endl;
	out<<intro<<endl;
	out<<skill<<endl;
	out<<full_screen<<endl;
	out<<speed<<endl;
	out<<fps<<endl;

	for(int i=0; i<18; i++)
	out<<keys[i]<<endl;
}
void Config::reset()
{
	const int k[18]={1,4,23,19,17,5,26,24,3,85,87,89,86,88,90,82,83,84};
	music=true;
	music_vol=2;
	intro=true;
	skill=2;
	full_screen=true;
	speed=3;
	fps=false;
	memcpy(keys, k, sizeof(int) * 18);
}
void Config::read_maxl()
{
	if(last_levelset=="default")
	{
		maxl=100;
		return;
	}
	else
	{
		int i=0;

		try
		{
			CL_DirectoryScanner scan;

			if(scan.scan((custom_path+"levels/"+last_levelset), "*.lvl"))
			{
				while(scan.next())
				{
					i++;
				}
			}
		}	
		catch(CL_Error err)
		{
			cout<<"no maps"<<endl;
		}
		
		maxl=i;
	}
}
int Config::get_maxl()
{
	return maxl;
}
